/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.soap.listener.outgoing;

import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axis2.AxisFault;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.objectweb.petals.component.framework.MEPConstants;
import org.objectweb.petals.ws.addressing.EndpointReference;
import org.objectweb.petals.ws.client.SoapClient;

/**
 * A {@link SoapClient} implementation that will be used to send Web Service
 * Notifications.
 *
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 *
 */
public class Axis2NotificationConsumerClient implements SoapClient {

    private Logger logger;

    /**
     * Creates a new instance of {@link Axis2NotificationConsumerClient}
     *
     */
    public Axis2NotificationConsumerClient(final Logger logger) {
        this.logger = logger;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.ws.client.SoapClient#send(org.objectweb.petals.ws.addressing.EndpointReference,
     *      org.objectweb.petals.ws.addressing.EndpointReference,
     *      java.lang.String, org.apache.axiom.om.OMElement,
     *      org.apache.axiom.om.OMElement[])
     */
    public SOAPEnvelope send(EndpointReference src, EndpointReference dest,
            String wsaAction, OMElement body, OMElement[] extraHeaders) {
        this.logger.fine("Sending SOAP message to " + dest.getAddress());

        SOAPEnvelope result = null;
        ServiceClient client = null;

        try {
            // create the service client
            client = Axis2Utils.createServiceClient(
                    MEPConstants.IN_OPTIONAL_OUT_PATTERN.value(), new QName(
                            wsaAction));

            // create options
            Options options = new Options();
            options.setTo(new org.apache.axis2.addressing.EndpointReference(
                    dest.getAddress().toString()));
            options.setAction(wsaAction);
            options.setFrom(new org.apache.axis2.addressing.EndpointReference(
                    src.getAddress().toString()));
            client.setOptions(options);

            // engaging addressing module
            client.engageModule(new QName("addressing"));

            // send request
            OMElement response = client.sendReceive(new QName(wsaAction), body);

        } catch (AxisFault e) {
            logger.warning(e.getMessage());
        } catch (MessagingException e) {
            logger.warning(e.getMessage());
        }

        return result;
    }
}
