/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.soap.listener.outgoing;

import java.io.StringReader;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.objectweb.petals.binding.soap.SoapComponentContext;
import org.objectweb.petals.binding.soap.listener.SUProperties;
import org.objectweb.petals.component.framework.util.Exchange;
import org.objectweb.petals.component.framework.util.Extensions;
import org.objectweb.petals.ws.fault.WsnFault;
import org.objectweb.petals.ws.notification.WsnManager;

/**
 * @author chamerling - eBM WebSourcing
 *
 */
public class WSNotifier extends AbstractJBIDispatcher {

    /**
     * Creates a new instance of {@link WSNotifier}
     *
     * @param soapContext
     * @param logger
     */
    public WSNotifier(SoapComponentContext soapContext, Logger logger) {
        super(soapContext, logger);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.binding.soap.listener.outgoing.JBIListenerDispatcher#process(org.objectweb.petals.component.common.util.MessageExchangeWrapper,
     *      org.objectweb.petals.common.descriptor.Extensions)
     */
    public void process(Exchange exchange,
            Extensions cdkExtensions) {

        // get the topic name (from extensions)
        String address = cdkExtensions.getPetalsExtensions().getProperty(
                SUProperties.ADDRESS);

        QName topicName = new QName(SUProperties
                .getTopicNameFromExtensionAddress(address));

        logger.fine("Posting message to topic : " + topicName);

        if (!"".equals(topicName)) {
            sendNotification(topicName, exchange);
        } else {
            logger
                    .warning("No topic name has been found, the message can not be delivered");
            // TODO : put a fault in the message exchange
        }
    }

    /**
     * Send a notification to all the topic subscribers
     *
     * @param topicName
     * @param exchange
     */
    protected void sendNotification(QName topicName,
            Exchange exchange) {

        WsnManager manager = soapContext.getWsnManager();

        try {
            OMElement notification = createNotificationContentFromJBIMessage(exchange);
            manager.publish(topicName, notification,
                    new Axis2NotificationConsumerClient(this.logger));
        } catch (WsnFault e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (XMLStreamException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.binding.soap.listener.outgoing.JBIListenerDispatcher#getDispatcherType()
     */
    public String getDispatcherType() {
        return "topic";
    }

    /**
     * Create the notification message from the JBI message exchange
     *
     * @param exchange
     * @return
     */
    public OMElement createNotificationContentFromJBIMessage(
            Exchange exchange) throws XMLStreamException {
        OMElement result = null;
        String contentString = null;

        try {
            contentString = exchange.getInMessageContentAsString();
        } catch (MessagingException e) {
            e.printStackTrace();
        }

        XMLStreamReader parser = XMLInputFactory.newInstance()
                .createXMLStreamReader(new StringReader(contentString));
        StAXOMBuilder builder = new StAXOMBuilder(parser);

        result = builder.getDocumentElement();
        return result;
    }
}
